_G.LDDG = _G.LDDG or {}
LDDG.Settings = LDDG.Settings or {}
LDDG.DefaultSettings = LDDG.DefaultSettings or {} 
LDDG.ModPath = ModPath
LDDG.SavePath = SavePath .. "LDDGOptions.txt"
LDDG.OptionsMenu = "LDDGOptionsMenu"
LDDG.HookFiles = {    
    ["lib/managers/hud/hudassaultcorner"] = "Hud/HudAssaultCorner.lua",     
    ["lib/managers/hud/hudplayerdowned"] = "Hud/HudPlayerDowned.lua",    
    ["lib/managers/hud/hudheisttimer"] = "Hud/HudHeistTimer.lua",
    ["lib/managers/hud/hudobjectives"] = "Hud/HudObjectives.lua",    
    ["lib/managers/hud/hudpresenter"] = "Hud/HudPresenter.lua",    
    ["lib/managers/hud/hudteammate"] = "Hud/HudTeammate.lua",
    ["lib/managers/hudmanagerpd2"] = "Hud/HudManager.lua",
    ["lib/managers/hud/hudtemp"] = "Hud/HudTemp.lua",
    ["lib/managers/hud/hudhint"] = "Hud/HudHint.lua",
}
LDDG.colors = LDDG.colors or {
     {color = Color(1, 1, 1), menu_name = "LDDG_HudColor"},     
     {color = Color(1, 1, 1), menu_name = "Color_White"},     
     {color = Color(1, 1, 0), menu_name = "Color_Yellow"},  
     {color = Color(1, 0, 0.2), menu_name = "Color_Red"},      
     {color = Color(0.2,1, 0), menu_name = "Color_Green"},
     {color = Color(0.2, 0.6 ,1 ), menu_name = "Color_Blue"},        
     {color = Color(1,0.7 ,0 ), menu_name = "Color_Orange"},
     {color = Color(1, 0.25, 0.7), menu_name = "Color_Pink"},                
     {color = Color(0, 0, 0), menu_name = "Color_Black"},                
     {color = Color(0.15, 0.15, 0.15), menu_name = "Color_Grey"},
     {color = Color(0, 1, 0.9), menu_name = "Color_Cyan"},
     {color = Color(0.5, 0, 1), menu_name = "Color_Purple"},
     {color = Color(0.6,0.2,0.1), menu_name = "Color_Brown"},
     {color = Color(0.7, 0.9, 0), menu_name = "Color_Lime"},
}
function LDDG:Save()
    local file = io.open( self.SavePath, "w+" )
    if file then
        file:write( json.encode( self.Settings ) )
        file:close()
    end
end
function LDDG:Load()
    local file = io.open( self.SavePath, "r" )
    if file then
        self.Settings = json.decode( file:read("*all") )
        file:close() 
    end
end
function LDDG:Update()
    LDDG.colors[1].color = LDDG.colors[math.min(self:GetSetting("HudColor") + 1, #self.colors)].color
    local updaters = {
        "_hud_assault_corner",        
        "_hud_player_downed",        
        "_hud_heist_timer",
        "_hud_objectives",        
        "_hud_presenter",
        "_hud_hint",
        "_hud_temp",
    }
    if managers.hud then
        managers.hud:UpdateLDDG()
        for _, hud in pairs(updaters) do
            if managers.hud[hud].UpdateLDDG then
                managers.hud[hud]:UpdateLDDG()
            end
        end
        for _, teammate in pairs(managers.hud._teammate_panels) do
            if teammate.UpdateLDDG then
                teammate:UpdateLDDG()
            end
        end
    end
end
function LDDG:Log( msg )
    log("[LDDGHUD] " .. msg)
end
function LDDG:GetSetting( setting )
    if LDDG.Settings[setting] == nil then
        self:Log("[ERROR] Setting: " .. tostring(setting) .." Doesn't exist!")
    end

    return LDDG.Settings[setting]
end
function LDDG:GetColor( setting )
    if LDDG.Settings[setting] and LDDG.colors[LDDG.Settings[setting]].color then
        return LDDG.colors[LDDG.Settings[setting]].color or Color.white
    else
        return Color.white
    end
end
if not LDDG.setup then  
    LDDG:Load()
    LDDG.DefaultSettings = {
        HudColor = 1,
        ObjectiveColor = 1,
        TeammatesColor = 1,       
        PresenterColor = 6,
        CarryBagColor = 1,         
        AssaultColor = 3,        
        HostageColor = 1,
        TimerColor = 1,
        CasingColor = 1,
        EscapeColor = 4,
        HintColor = 1,
        HudScale = 1,        
        FixedAmmoTotal = false,
        RealTime = false,
    }
    for option, value in pairs(LDDG.DefaultSettings) do
        if LDDG.Settings[option] == nil then
            LDDG.Settings[option] = value
        end
    end
    LDDG.colors[1].color = LDDG.colors[math.min(LDDG.Settings.HudColor + 1, #LDDG.colors)].color    
    LDDG.setup = true
    Input:keyboard():add_trigger(Idstring("f1"), function()
        LDDG.Settings.RealTime = not LDDG.Settings.RealTime 
        LDDG:Save()
    end)
    LDDG:Log("All Settings Are Loaded!")
end
if RequiredScript then
	local requiredScript = RequiredScript:lower()
	if LDDG.HookFiles[requiredScript] then
 		dofile( ModPath .. LDDG.HookFiles[requiredScript] )
	end	
end
if Hooks then
    Hooks:Add("MenuManagerSetupCustomMenus", "MenuManagerSetupCustomMenusCompactHUD", function( menu_manager, nodes )
        MenuHelper:NewMenu( LDDG.OptionsMenu )
    end)
	Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_LDDG", function(loc)
		loc:load_localization_file( LDDG.ModPath .. "/Loc/EN.txt" )	
	end)
    Hooks:Add("MenuManagerPopulateCustomMenus", "PopulateLDDGMenu", function( menu_manager, nodes )
        MenuCallbackHandler.LDDG_ValueClbk = function(self, item)
            LDDG.Settings[item._parameters.name] = item:value()
            LDDG:Save()
            LDDG:Update()     
        end             
        MenuCallbackHandler.LDDG_ToggleClbk = function(self, item)
            LDDG.Settings[item._parameters.name] = item:value() == "on" and true or false
            LDDG:Save()
            LDDG:Update()     
        end         
        MenuCallbackHandler.LDDG_ResetOptions = function(self, item)
            local menu = MenuHelper:GetMenu(LDDG.OptionsMenu)
            for _, item in pairs(menu._items_list) do
                MenuHelper:ResetItemsToDefaultValue(item, {[item._parameters.name] = true}, LDDG.DefaultSettings[item._parameters.name])
            end
        end                         
        local colors = {}
        for _, ColorTable in pairs(LDDG.colors) do
            table.insert(colors, ColorTable.menu_name)
        end
        local _colors = clone(colors) 
        table.remove(_colors, 1)
        local i = 1
        for setting, v in pairs(LDDG.Settings) do
            if setting:match("Color") then
                i = i + 1
                MenuHelper:AddMultipleChoice({
                    id = setting,
                    title = "LDDG_" .. setting,
                    callback = "LDDG_ValueClbk",
                    items = setting == "HudColor" and _colors or colors,
                    menu_id = LDDG.OptionsMenu,
                    value = v,
                    priority = 999 - i,
                }) 
            end
        end
        MenuHelper:AddSlider({
            id = "HudScale",
            title = "LDDG_HudScale",
            desc = "LDDG_HudScale_desc",
            callback = "LDDG_ValueClbk",
            max = 2, 
            min = 0.6,
            value = LDDG.Settings.HudScale,
            menu_id = LDDG.OptionsMenu,
            show_value = true,
            priority = 980 
        })             
        MenuHelper:AddToggle({
            id = "FixedAmmoTotal",
            title = "LDDG_FixedAmmoTotal",
            desc = "LDDG_FixedAmmoTotal_desc",
            callback = "LDDG_ToggleClbk",
            value = LDDG.Settings.FixedAmmoTotal,
            menu_id = LDDG.OptionsMenu,
            priority = 970 
        })            
        MenuHelper:AddButton({
            id = "LDDG_ResetOptions",
            title = "LDDG_ResetOptions",
            desc = "LDDG_ResetOptions_desc",
            callback = "LDDG_ResetOptions",
            menu_id = LDDG.OptionsMenu,
            priority = 0 
        })              
    end)
    Hooks:Add("MenuManagerBuildCustomMenus", "MenuManagerBuildCustomMenusCompactHUD", function(menu_manager, nodes)
        nodes[LDDG.OptionsMenu] = MenuHelper:BuildMenu( LDDG.OptionsMenu )
        MenuHelper:AddMenuItem( MenuHelper.menus.lua_mod_options_menu, LDDG.OptionsMenu, "LDDGHUD_options", "LDDGHUD_options_desc", 1 )
    end)
end
if CriminalsManager then
    Hooks:PostHook(CriminalsManager, "add_character", "set_players_portrait", function(self)
        for _, teammate in pairs(managers.hud._teammate_panels) do
            if teammate.set_player_portrait then
                teammate:set_player_portrait()
            end
        end
    end)
end