local orig = HUDBGBox_create
function HUDBGBox_create(panel, params, config)
	config = config or {}
	config.color = Color.white:with_alpha(0)
	config.bg_color = Color.white:with_alpha(0)	
	local box_panel = orig(panel, params, config)
	box_panel:child("left_top"):hide()
	box_panel:child("left_bottom"):hide()
	box_panel:child("right_top"):hide()
	box_panel:child("right_bottom"):hide()
	return box_panel
end
Hooks:PostHook(HUDObjectives, "init", "hudobjectives_lddg", function(self, hud, ...)	
	self._hud_panel = hud.panel
	if self._hud_panel:child("objectives_panel") then
		self._hud_panel:remove(self._hud_panel:child("objectives_panel"))
	end
	local objectives_panel = self._hud_panel:panel({
		visible = false,
		name = "objectives_panel",
		h = 100,
		w = 500,
		valign = "top"
	})
	local icon_objectivebox = objectives_panel:bitmap({
		halign = "left",
		valign = "top",
		name = "icon_objectivebox",
		color = LDDG:GetColor("ObjectiveColor"),
		texture = "guis/textures/pd2/hud_icon_objectivebox",
		w = 24,
		h = 24
	})	
	self._bg_box = HUDBGBox_create( objectives_panel,{
		w = 200,
		h = 30,
		x = 26
	})
	local objective_text = objectives_panel:text({ 
		name = "objective_text", 
		layer = 2, 
		color = LDDG:GetColor("ObjectiveColor"), 
		font_size = 24, 
		font = "fonts/font_large_mf" , 
		x = self._bg_box:x() + 8, 
		y = 4, 
		vertical = "top"
    })
	local amount_text = objectives_panel:text({ 
    	name = "amount_text",
    	layer = 2, 
    	color = LDDG:GetColor("ObjectiveColor"), 
	    text="1/4", 
    	font_size = 24, 
    	font = "fonts/font_large_mf", 
	    y = 4, 
	    vertical = "top"
    })	
	self:UpdateLDDG()
end)
function HUDObjectives:UpdateLDDG()
	local scale = LDDG:GetSetting("HudScale")
	local objectives_panel = self._hud_panel:child( "objectives_panel" )
	local amount_text = objectives_panel:child("amount_text")
	local objective_text = objectives_panel:child("objective_text")
	local icon_objectivebox = objectives_panel:child("icon_objectivebox")
	icon_objectivebox:configure({
		w = 24 * scale,
		h = 24 * scale,
		color = LDDG:GetColor("ObjectiveColor"),
	})
	objective_text:configure({
		font = "fonts/font_large_mf",
		x = self._bg_box:x() + 8,
		font_size = 24 * scale,
		h = 24 * scale,
		color = LDDG:GetColor("ObjectiveColor"),
	})	
	amount_text:configure({
		font = "fonts/font_large_mf",
		font_size = 24 * scale,
		h = 24 * scale,
		color = LDDG:GetColor("ObjectiveColor"),
	})
	local _,_,objective_w,_ = objective_text:text_rect()
	objective_text:set_left(icon_objectivebox:right() + 4)
	objective_text:set_center_y(icon_objectivebox:center_y())
	amount_text:set_center_y(icon_objectivebox:center_y())
	amount_text:set_x( objective_text:x() + 5 + objective_w )	
end
local box_speed = 1000
function hudbox_animate_open_right( panel, wait_t, target_w, done_cb, color, icon  )
	panel:set_visible( false )
	panel:set_w( 0 )
	if wait_t then
		wait( wait_t )
	end
	panel:set_visible( true )
	local speed = box_speed
	local TOTAL_T = target_w/speed
	local t = TOTAL_T
	while t > 0 do
		local dt = coroutine.yield()
		t = t - dt
		panel:set_w( (1 - t/TOTAL_T) * target_w )
	end
	
	panel:set_w( target_w )
	if done_cb then
		done_cb()
	end
end
function hudbox_animate_open_left( panel, wait_t, target_w, done_cb, color, icon )
	panel:set_visible( false )
	local right = panel:right()
	panel:set_w( 0 )
	panel:set_right( right )
	if wait_t then
		wait( wait_t )
	end
	panel:set_visible( true )
	local speed = box_speed
	local TOTAL_T = target_w/speed
	local t = TOTAL_T
	while t > 0 do
		local dt = coroutine.yield()
		t = t - dt
		panel:set_w( (1 - t/TOTAL_T) * target_w )
		panel:set_right( right )
	end
	
	panel:set_w( target_w )
	panel:set_right( right )
	if done_cb then
		done_cb()
	end
end
function set_alpha( o, a, ct )
	local t = 0
	local target = ct or 0.5
	local ca = o:alpha()
	while t < target do
		t = t + coroutine.yield()
		local n = math.sin( t * 200 ) 
		o:set_alpha(math.lerp(ca, a, n))
	end
	o:set_alpha(a)
end
function set_color( o, c, ct )
	local t = 0
	local target = ct or 0.5
	local cc = o:color()
	while t < target do
		t = t + coroutine.yield()
		local n = math.sin( t * 200 ) 
		o:set_color(Color(math.lerp(cc.r, c.r, n), math.lerp(cc.g, c.g, n), math.lerp(cc.b, c.b, n)))
	end
	o:set_color(c)
end
function HUDObjectives:activate_objective( data )
	local objectives_panel = self._hud_panel:child( "objectives_panel" )
	local objective_text = objectives_panel:child( "objective_text" )
	local amount_text = objectives_panel:child( "amount_text" )
	local icon_objectivebox = objectives_panel:child("icon_objectivebox")

	self._active_objective_id = data.id
	objective_text:set_text( utf8.to_upper( data.text ) )
	local _,_,objective_w,_ = objective_text:text_rect()
	local _,_,amount_w,_ = amount_text:text_rect()	   
	amount_text:set_visible(data.amount)

	objectives_panel:stop()
	objectives_panel:set_alpha(0)
	objectives_panel:set_visible( true )	
	objectives_panel:animate(callback(nil, _G, "set_alpha"), 1)
    amount_size = 26		
	if data.amount then 
		self:update_amount_objective( data )
		amount_size = amount_size + amount_w 
	end
	self._bg_box:stop()
	self._bg_box:animate(callback( nil, _G, "hudbox_animate_open_right" ), 0.66, objective_w + amount_size, callback( self, self, "open_right_done", data.amount and true or false ), Color(), icon_objectivebox)
   
	amount_text:set_x( objective_text:x() + 5 + objective_w )	
	if data.amount then
		self:update_amount_objective( data )
	end
end
function HUDObjectives:update_amount_objective(data)
	if data.id ~= self._active_objective_id then
		return
	end
	local current = data.current_amount or 0
	local amount = data.amount
	local objectives_panel = self._hud_panel:child("objectives_panel")
	objectives_panel:child("amount_text"):set_text(current .. "/" .. amount)
end
function HUDObjectives:complete_objective(data)
	if data.id ~= self._active_objective_id then
		return
	end
	local objectives_panel = self._hud_panel:child("objectives_panel")
	objectives_panel:stop()
	objectives_panel:animate(callback(nil, _G, "set_alpha"), 0)
end
function HUDObjectives:open_right_done(uses_amount)
	local objectives_panel = self._hud_panel:child("objectives_panel")
	local objective_text = objectives_panel:child("objective_text")
	local amount_text = objectives_panel:child("amount_text")
	amount_text:set_visible(uses_amount)
end
function HUDObjectives:remind_objective(id)
	if id ~= self._active_objective_id then
		return
	end
end